@@ -13,18 +13,6 @@ class TabulatorException(Exception):
     pass
 
 
-class IOError(TabulatorException):
-    """Local loading error
-    """
-    pass
-
-
-class HTTPError(IOError):
-    """Remote loading error
-    """
-    pass
-
-
 class SourceError(TabulatorException):
     """The source file could not be parsed correctly.
     """
@@ -49,7 +37,31 @@ class EncodingError(TabulatorException):
     pass
 
 
+class CompressionError(TabulatorException):
+    """Compression error
+    """
+    pass
+
+
 # Deprecated
 
 OptionsError = TabulatorException
 ResetError = TabulatorException
+
+
+class IOError(SchemeError):
+    """Local loading error
+    """
+    pass
+
+
+class LoadingError(IOError):
+    """Local loading error
+    """
+    pass
+
+
+class HTTPError(LoadingError):
+    """Remote loading error
+    """
+    pass
