@@ -66,11 +66,11 @@ class Stream(object):
             To provide a regex pattern use  `{'type'\\: 'regex', 'value'\\: '^#'}`
             For example\\: `skip_rows=[1, '# comment', {'type'\\: 'regex', 'value'\\: '^# (regex|comment)'}]`
 
-        pick_fields (str[]):
+        pick_fields (List[Union[int, str]], optional):
             When passed, ignores all columns with headers
             that the given list DOES NOT include
 
-        skip_fields (str[]):
+        skip_fields (List[Union[int, str]], optional):
             When passed, ignores all columns with headers
             that the given list includes. If it contains an empty string it will skip
             empty headers
@@ -96,6 +96,16 @@ class Stream(object):
             as `<passed-value>.join(header1_1, header1_2)`
             Defaults to ' ' (space).
 
+        multiline_headers_duplicates (bool, optional):
+            By default tabulator will exclude a cell of a miltilne header from joining
+            if it's exactly the same as the previous seen value in this field.
+            Enabling this option will force duplicates inclusion
+            Defaults to False.
+
+        hashing_algorithm (func, optional):
+            It supports: md5, sha1, sha256, sha512
+            Defaults to sha256
+
         force_strings (bool, optional):
             When True, casts all data to strings.
             Defaults to False.
@@ -145,26 +155,26 @@ class Stream(object):
                  ignore_listed_headers=None,
                  ignore_not_listed_headers=None,
                  multiline_headers_joiner=' ',
+                 multiline_headers_duplicates=False,
+                 hashing_algorithm='sha256',
                  force_strings=False,
                  force_parse=False,
-                 pick_rows=None,
-                 skip_rows=None,
-                 pick_fields=None,
-                 skip_fields=None,
                  pick_columns=None,
                  skip_columns=None,
+                 pick_fields=None,
+                 skip_fields=None,
+                 limit_fields=None,
+                 offset_fields=None,
+                 pick_rows=None,
+                 skip_rows=None,
+                 limit_rows=None,
+                 offset_rows=None,
                  post_parse=[],
                  custom_loaders={},
                  custom_parsers={},
                  custom_writers={},
                  **options):
 
-        # Validate compression
-        if compression:
-            if compression not in config.SUPPORTED_COMPRESSION:
-                message = 'Not supported compression "%s"' % compression
-                raise exceptions.CompressionError(message)
-
         # Translate aliases
         if pick_fields is not None:
             pick_columns = pick_fields
@@ -248,9 +258,15 @@ class Stream(object):
         self.__ignore_listed_headers = ignore_listed_headers
         self.__ignore_not_listed_headers = ignore_not_listed_headers
         self.__multiline_headers_joiner = multiline_headers_joiner
+        self.__multiline_headers_duplicates = multiline_headers_duplicates
         self.__ignored_headers_indexes = []
+        self.__hashing_algorithm = hashing_algorithm
         self.__force_strings = force_strings
         self.__force_parse = force_parse
+        self.__limit_fields = limit_fields
+        self.__offset_fields = offset_fields
+        self.__limit_rows = limit_rows
+        self.__offset_rows = offset_rows
         self.__post_parse = copy(post_parse)
         self.__custom_loaders = copy(custom_loaders)
         self.__custom_parsers = copy(custom_parsers)
@@ -299,6 +315,15 @@ class Stream(object):
         source = self.__source
         options = copy(self.__options)
 
+        # Programming error assertions
+        assert self.__hashing_algorithm in config.SUPPORTED_HASHING_ALGORITHMS
+
+        # Validate compression
+        if self.__compression:
+            if self.__compression not in config.SUPPORTED_COMPRESSION:
+                message = 'Not supported compression "%s"' % self.__compression
+                raise exceptions.CompressionError(message)
+
         # Get scheme and format if not already given
         compression = None
         if self.__scheme is None or self.__format is None:
@@ -368,7 +393,7 @@ class Stream(object):
 
         # Attach stats to the loader
         if getattr(self.__loader, 'attach_stats', None):
-            self.__stats = {'size': 0, 'hash': ''}
+            self.__stats = {'size': 0, 'hash': '', 'hashing_algorithm': self.__hashing_algorithm}
             getattr(self.__loader, 'attach_stats')(self.__stats)
 
         # Initiate parser
@@ -405,7 +430,7 @@ class Stream(object):
         self.__actual_scheme = scheme
         self.__actual_format = format
         self.__actual_encoding = self.__parser.encoding
-        self.__actual_compression = compression or 'no'
+        self.__actual_compression = compression
 
         return self
 
@@ -463,7 +488,37 @@ class Stream(object):
             str: scheme
 
         """
-        return self.__actual_scheme
+        return self.__actual_scheme or 'inline'
+
+    @property
+    def format(self):
+        """Path's format
+
+        # Returns
+            str: format
+
+        """
+        return self.__actual_format or 'inline'
+
+    @property
+    def encoding(self):
+        """Stream's encoding
+
+        # Returns
+            str: encoding
+
+        """
+        return self.__actual_encoding or 'no'
+
+    @property
+    def compression(self):
+        """Stream's compression ("no" if no compression)
+
+        # Returns
+            str: compression
+
+        """
+        return self.__actual_compression or 'no'
 
     @property
     def fragment(self):
@@ -477,36 +532,6 @@ class Stream(object):
             return getattr(self.__parser, 'fragment', None)
         return None
 
-    @property
-    def format(self):
-        """Path's format
-
-        # Returns
-            str: format
-
-        """
-        return self.__actual_format
-
-    @property
-    def encoding(self):
-        """Stream's encoding
-
-        # Returns
-            str: encoding
-
-        """
-        return self.__actual_encoding
-
-    @property
-    def compression(self):
-        """Stream's compression ("no" if no compression)
-
-        # Returns
-            str: compression
-
-        """
-        return self.__actual_compression
-
     @property
     def size(self):
         """Returns the BYTE count of the read chunks if available
@@ -520,10 +545,11 @@ class Stream(object):
 
     @property
     def hash(self):
-        """Returns the SHA256 hash of the read chunks if available
+        """Returns the SHA256 (or according to the "hashing_algorithm" parameter)
+        hash of the read chunks if available
 
         # Returns
-            str/None: SHA256 hash
+            str/None: bytes hash
 
         """
         if self.__stats:
@@ -558,6 +584,10 @@ class Stream(object):
                         self.__field_positions.append(index + 1)
         return self.__field_positions
 
+    @property
+    def hashing_algorithm(self):
+        return self.__hashing_algorithm
+
     def iter(self, keyed=False, extended=False):
         """Iterate over the rows.
 
@@ -600,8 +630,17 @@ class Stream(object):
 
         # Yield rows from iterator
         try:
+            count = 0
             for row_number, headers, row in iterator:
                 if row_number > self.__row_number:
+                    count += 1
+                    if self.__limit_rows or self.__offset_rows:
+                        offset = self.__offset_rows or 0
+                        limit = self.__limit_rows + offset if self.__limit_rows else None
+                        if offset and count <= offset:
+                            continue
+                        if limit and count > limit:
+                            break
                     self.__row_number = row_number
                     if extended:
                         yield (row_number, headers, row)
@@ -717,6 +756,7 @@ class Stream(object):
             raise exceptions.TabulatorException(message)
 
         # Get headers from data
+        last_merged = {}
         keyed_source = False
         for row_number, headers, row in self.__sample_extended_rows:
             keyed_source = keyed_source or headers is not None
@@ -726,16 +766,18 @@ class Stream(object):
                     headers[index] = six.text_type(header).strip()
             if row_number == self.__headers_row:
                 self.__headers = headers
+                last_merged = {index: header for index, header in enumerate(headers)}
             if row_number > self.__headers_row:
                 for index in range(0, len(self.__headers)):
                     if len(headers) > index and headers[index] is not None:
                         if not self.__headers[index]:
                             self.__headers[index] = headers[index]
                         else:
-                            # https://github.com/frictionlessdata/tabulator-py/issues/292
-                            if (not self.__options.get('fill_merged_cells') or
-                                    not self.__headers[index].endswith(headers[index])):
-                                self.__headers[index] += self.__multiline_headers_joiner + headers[index]
+                            if (self.__multiline_headers_duplicates or
+                                    last_merged.get(index) != headers[index]):
+                                self.__headers[index] += (
+                                    self.__multiline_headers_joiner + headers[index])
+                        last_merged[index] = headers[index]
             if row_number == self.__headers_row_last:
                 break
 
@@ -755,22 +797,59 @@ class Stream(object):
                     if (header in self.__ignore_listed_headers or
                             index + 1 in self.__ignore_listed_headers):
                         ignore = True
+                    # Regex
+                    for item in self.__ignore_listed_headers:
+                        if isinstance(item, dict) and item.get('type') == 'regex':
+                            if bool(re.search(item['value'], header)):
+                                ignore = True
                 # Ignore not-listed headers
                 if self.__ignore_not_listed_headers is not None:
                     if (header not in self.__ignore_not_listed_headers and
                             index + 1 not in self.__ignore_not_listed_headers):
                         ignore = True
+                    # Regex
+                    for item in self.__ignore_not_listed_headers:
+                        if isinstance(item, dict) and item.get('type') == 'regex':
+                            if bool(re.search(item['value'], header)):
+                                ignore = False
                 # Add to the list and skip
                 if ignore:
                     self.__ignored_headers_indexes.append(index)
                     continue
                 self.__headers.append(header)
-            self.__ignored_headers_indexes = sorted(self.__ignored_headers_indexes, reverse=True)
+            self.__ignored_headers_indexes = list(sorted(self.__ignored_headers_indexes, reverse=True))
+
+        # Limit/offset fields
+        if self.__limit_fields or self.__offset_fields:
+            ignore = []
+            headers = []
+            min = self.__offset_fields or 0
+            max = self.__limit_fields + min if self.__limit_fields else len(self.__headers)
+            for position, header in enumerate(self.__headers, start=1):
+                if position <= min:
+                    ignore.append(position - 1)
+                    continue
+                if position > max:
+                    ignore.append(position - 1)
+                    continue
+                headers.append(header)
+            for index in ignore:
+                if index not in self.__ignored_headers_indexes:
+                    self.__ignored_headers_indexes.append(index)
+            self.__ignored_headers_indexes = list(sorted(self.__ignored_headers_indexes, reverse=True))
+            self.__headers = headers
 
         # Remove headers from data
         if not keyed_source:
             del self.__sample_extended_rows[:self.__headers_row_last]
 
+        # Stringify headers
+        if isinstance(self.__headers, list):
+            str_headers = []
+            for header in self.__headers:
+                str_headers.append(six.text_type(header) if header is not None else '')
+            self.__headers = str_headers
+
     def __detect_html(self):
 
         # Prepare text
@@ -880,12 +959,12 @@ class Stream(object):
 
             # Pick by pattern
             for pattern in self.__pick_rows_by_patterns:
-                if bool(pattern.match(cell)):
+                if bool(pattern.search(cell)):
                     return False
 
             # Pick by comment
             for comment in filter(None, self.__pick_rows_by_comments):
-                if cell.startswith(comment):
+                if six.text_type(cell).startswith(comment):
                     return False
 
             # Default
@@ -912,12 +991,12 @@ class Stream(object):
 
             # Skip by pattern
             for pattern in self.__skip_rows_by_patterns:
-                if bool(pattern.match(cell)):
+                if bool(pattern.search(cell)):
                     return True
 
             # Skip by comment
             for comment in filter(None, self.__skip_rows_by_comments):
-                if cell.startswith(comment):
+                if six.text_type(cell).startswith(comment):
                     return True
 
             # Default
